#include <Wire.h>
#include <Adafruit_PWMServoDriver.h> // Librería shield servos
#include "WiiChuck.h" //Libreria WiiChuck
#include <math.h>

// De esta manera le llamamos a la direccion por defecto 0x40
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();
// Se puede llamar a la direccion que se quiera de esta forma
// Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(0x41);

// Dependiendo del tipo de servo el pulso máximo y mínimo es diferente
// Lo preferible es que sea el máximo posible sin llegar a hacer tope

#define SERVOMIN  150 // Minimo (sobre 4096)
#define SERVOMAX  550 // Maximo (sobre 4096)


const unsigned short h1 = 0; //Hombro extremidad 1
const unsigned short p1 = 1; //Pata extremidad 1
const unsigned short h2 = 4; //Hombro extremidad 2
const unsigned short p2 = 5; //Pata extremidad 2
const unsigned short h3 = 8; //Hombro extremidad 3
const unsigned short p3 = 9; //Pata extremidad 3
const unsigned short h4 = 12; //Hombro extremidad 4
const unsigned short p4 = 13; //Pata extremidad 4

WiiChuck chuck = WiiChuck(); //Creamos la instancia de WiiChuck

void setup()
{
  Serial.begin(115200); //Configuramos la velocidad del puerto serie
  chuck.begin(); //Iniciamos WiiChuck
  pwm.begin();
  pwm.setPWMFreq(60);  // Los servos funcionan a ~60 Hz
  depie();
}

void loop()
{
  delay(100);
  chuck.update(); //Actualizamos los valores

  //Imprimimos los valores por pantalla

  //Joystick
  Serial.print("Eje X: ");
  Serial.print(chuck.readJoyX()); //Eje X del Joystick
  Serial.print(" ");
  Serial.print("Eje Y: ");
  Serial.print(chuck.readJoyY()); //Eje X del Joystick
  Serial.print(" ");

  //Boton Z
  if (chuck.buttonZ)
    Serial.print("Z");
  else
    Serial.print("-");

  Serial.print(" ");

  //Boton C
  if (chuck.buttonC)
    Serial.println("C");
  else
    Serial.println("-");

  //Acelerometros
  Serial.print("Lecturas: ");
  Serial.print((int)chuck.readAccelX()); //Lectura eje X
  Serial.print(" ");
  Serial.print((int)chuck.readAccelY()); //Lectura eje Y
  Serial.print(" ");
  Serial.println((int)chuck.readAccelZ()); //Lectura eje Z

  //Movimientos
  if (chuck.readJoyY() < 100 && chuck.readJoyY() > -100 && chuck.readJoyX() < 100 && chuck.readJoyX() > -100 && !chuck.buttonC && !chuck.buttonZ && chuck.readAccelY() > -125 && chuck.readAccelY() < 125 && chuck.readAccelX() > -125 && chuck.readAccelX() < 125)
    depie();

  //Joystick  
  if (chuck.readJoyX() > 100)
    girarderecha();
  if (chuck.readJoyX() < -100)
    girarizquierda();
  if (chuck.readJoyY() > 100)
    andar();
  if (chuck.readJoyY() < -100)
    atras();

  //Botones
  if (chuck.buttonC)
    tumbar();

  if (chuck.buttonZ)
    hola();

    // Acelerometro
  if (chuck.readAccelY() < -125)
    sentar();

  if (chuck.readAccelY() > 125)
    inclinar();

  if (chuck.readAccelX() < -125)
    inclinarizq();

  if (chuck.readAccelX() > 125)
    inclinarder();
}

// Funcion para tumbar el robot
void tumbar()
{
  pwm.setPWM(h1, 0, 350);
  pwm.setPWM(h2, 0, 350);
  pwm.setPWM(h3, 0, 350);
  pwm.setPWM(h4, 0, 350);

  pwm.setPWM(p1, 0, 350);
  pwm.setPWM(p2, 0, 350);
  pwm.setPWM(p3, 0, 350);
  pwm.setPWM(p4, 0, 350);
}

// Funcion para poner de pie el robot
void depie()
{
  pwm.setPWM(h1, 0, 350);
  pwm.setPWM(h2, 0, 350);
  pwm.setPWM(h3, 0, 350);
  pwm.setPWM(h4, 0, 350);

  pwm.setPWM(p1, 0, 500);
  pwm.setPWM(p2, 0, 200);
  pwm.setPWM(p3, 0, 500);
  pwm.setPWM(p4, 0, 200);
}

// Funcion para sentar el robot
void sentar()
{
  pwm.setPWM(h1, 0, 300);
  pwm.setPWM(h2, 0, 400);
  pwm.setPWM(h3, 0, 350);
  pwm.setPWM(h4, 0, 350);

  pwm.setPWM(p1, 0, 350);
  pwm.setPWM(p2, 0, 350);
  pwm.setPWM(p3, 0, 500);
  pwm.setPWM(p4, 0, 200);
}

// Funcion para inclinar hacia delante el robot
void inclinar()
{
  pwm.setPWM(h1, 0, 350);
  pwm.setPWM(h2, 0, 350);
  pwm.setPWM(h3, 0, 300);
  pwm.setPWM(h4, 0, 400);

  pwm.setPWM(p1, 0, 500);
  pwm.setPWM(p2, 0, 200);
  pwm.setPWM(p3, 0, 350);
  pwm.setPWM(p4, 0, 350);
}

// Funcion para inclinar hacia la derecha el robot
void inclinarder()
{
  pwm.setPWM(h1, 0, 400);
  pwm.setPWM(h2, 0, 350);
  pwm.setPWM(h3, 0, 350);
  pwm.setPWM(h4, 0, 300);

  pwm.setPWM(p1, 0, 350);
  pwm.setPWM(p2, 0, 200);
  pwm.setPWM(p3, 0, 500);
  pwm.setPWM(p4, 0, 350);
}

// Funcion para inclinar hacia la izquierda el robot
void inclinarizq()
{
  pwm.setPWM(h1, 0, 350);
  pwm.setPWM(h2, 0, 300);
  pwm.setPWM(h3, 0, 400);
  pwm.setPWM(h4, 0, 350);

  pwm.setPWM(p1, 0, 500);
  pwm.setPWM(p2, 0, 350);
  pwm.setPWM(p3, 0, 350);
  pwm.setPWM(p4, 0, 200);
}


// Funcion para que salude el robot
void hola()
{
  //posicionamos el robot
  pwm.setPWM(h1, 0, 350);
  pwm.setPWM(h2, 0, 250);
  pwm.setPWM(h3, 0, 350);
  pwm.setPWM(h4, 0, 450);

  pwm.setPWM(p1, 0, 400);
  pwm.setPWM(p2, 0, 200);
  pwm.setPWM(p3, 0, 550);
  pwm.setPWM(p4, 0, 200);
  delay (500);



  pwm.setPWM(p3, 0, 250);
  delay (500);

  for (int i = 0; i < 3; i++)
  {
    pwm.setPWM(h3, 0, 250);
    delay (500);
    pwm.setPWM(h3, 0, 450);
    delay (500);
  }
  depie();
}

// Funcion para que ande el robot
void andar()
{

  pwm.setPWM(p1, 0, 450);
  pwm.setPWM(p3, 0, 450);
  delay (100);

  for (int i = 0; i < 3; i++)
  {
    pwm.setPWM(h1, 0, 400);
    pwm.setPWM(h3, 0, 300);
    delay (100);

    pwm.setPWM(p1, 0, 500);
    pwm.setPWM(p3, 0, 500);
    delay (100);

    pwm.setPWM(p2, 0, 250);
    pwm.setPWM(p4, 0, 250);
    delay(100);

    pwm.setPWM(h1, 0, 350);
    pwm.setPWM(h3, 0, 350);
    delay(100);

    pwm.setPWM(h2, 0, 300);
    pwm.setPWM(h4, 0, 400);
    delay (100);

    pwm.setPWM(p2, 0, 200);
    pwm.setPWM(p4, 0, 200);
    delay(100);

    pwm.setPWM(p1, 0, 450);
    pwm.setPWM(p3, 0, 450);
    delay (100);

    pwm.setPWM(h2, 0, 350);
    pwm.setPWM(h4, 0, 350);
    delay(100);
  }
  depie();
}

// Funcion para que ande el robot hacia atras
void atras()
{

  pwm.setPWM(p1, 0, 450);
  pwm.setPWM(p3, 0, 450);
  delay (100);

  for (int i = 0; i < 3; i++)
  {
    pwm.setPWM(h1, 0, 300);
    pwm.setPWM(h3, 0, 400);
    delay (100);

    pwm.setPWM(p1, 0, 500);
    pwm.setPWM(p3, 0, 500);
    delay (100);

    pwm.setPWM(p2, 0, 250);
    pwm.setPWM(p4, 0, 250);
    delay(100);

    pwm.setPWM(h1, 0, 350);
    pwm.setPWM(h3, 0, 350);
    delay(100);

    pwm.setPWM(h2, 0, 400);
    pwm.setPWM(h4, 0, 300);
    delay (100);

    pwm.setPWM(p2, 0, 200);
    pwm.setPWM(p4, 0, 200);
    delay(100);

    pwm.setPWM(p1, 0, 450);
    pwm.setPWM(p3, 0, 450);
    delay (100);

    pwm.setPWM(h2, 0, 350);
    pwm.setPWM(h4, 0, 350);
    delay(100);
  }
  depie();
}

// Funcion para que gire el robot a la derecha
void girarderecha()
{

  pwm.setPWM(p1, 0, 450);
  pwm.setPWM(p3, 0, 450);
  delay (100);

  for (int i = 0; i < 3; i++)
  {
    pwm.setPWM(h1, 0, 300);
    pwm.setPWM(h3, 0, 300);
    delay (100);

    pwm.setPWM(p1, 0, 500);
    pwm.setPWM(p3, 0, 500);
    delay (100);

    pwm.setPWM(p2, 0, 250);
    pwm.setPWM(p4, 0, 250);
    delay(100);

    pwm.setPWM(h1, 0, 350);
    pwm.setPWM(h3, 0, 350);
    delay(100);

    pwm.setPWM(h2, 0, 300);
    pwm.setPWM(h4, 0, 300);
    delay (100);

    pwm.setPWM(p2, 0, 200);
    pwm.setPWM(p4, 0, 200);
    delay(100);

    pwm.setPWM(p1, 0, 450);
    pwm.setPWM(p3, 0, 450);
    delay (100);

    pwm.setPWM(h2, 0, 350);
    pwm.setPWM(h4, 0, 350);
    delay(100);
  }
  depie();
}

// Funcion para que gire el robot hacia la izquierda
void girarizquierda()
{

  pwm.setPWM(p1, 0, 450);
  pwm.setPWM(p3, 0, 450);
  delay (100);

  for (int i = 0; i < 3; i++)
  {
    pwm.setPWM(h1, 0, 400);
    pwm.setPWM(h3, 0, 400);
    delay (100);

    pwm.setPWM(p1, 0, 500);
    pwm.setPWM(p3, 0, 500);
    delay (100);

    pwm.setPWM(p2, 0, 250);
    pwm.setPWM(p4, 0, 250);
    delay(100);

    pwm.setPWM(h1, 0, 350);
    pwm.setPWM(h3, 0, 350);
    delay(100);

    pwm.setPWM(h2, 0, 400);
    pwm.setPWM(h4, 0, 400);
    delay (100);

    pwm.setPWM(p2, 0, 200);
    pwm.setPWM(p4, 0, 200);
    delay(100);

    pwm.setPWM(p1, 0, 450);
    pwm.setPWM(p3, 0, 450);
    delay (100);

    pwm.setPWM(h2, 0, 350);
    pwm.setPWM(h4, 0, 350);
    delay(100);
  }
  depie();
}
